* -------------------------------
* 	LPS Example
*
* 	Switch
*
*	Language: MC68K-ASM1
*	Style: plain MC68000
* 	Version: Ref2-m68k
* -------------------------------

* Reference Plain MC68000 Solution 2

* Questa soluzione mostra come tradurre in ASM
* un programma C che contiene un'istruzione switch
* mediante la tecnica della "Jump Table"

* Questo esempio e' realizzato per l'assembler ASM-One V1.48

* Variabili
; il tipo int viene memorizzato con long
	org	$4000
a:	dc.l	4
b:	dc.l	10
c:	dc.l	20
x:	dc.l	5

* Jump Table: un array di indirizzi di istruzioni
jump_table:
	dc.l	sw_case_m1,sw_case_0,sw_default
	dc.l	sw_case_2,sw_case_3,sw_case_4

* Programma

	org	$2000
inizio_codice:

switch_beg:

	* copia x in d0, per le successive istruzioni
	move.l	x,d0

	* se il valore della variabile e' esterno
	; all'intervallo dei valori delle etichette case,
	; salta al caso di default
	cmp.l	#-1,d0
	blt	sw_default
	cmp.l	#4,d0
	bgt	sw_default

	; trasformo d0 in un indice in Jump Table, aumentandolo di 1
	add.l	#1,d0

	; leggo da Jump Table l' indirizzo dell'istruzione destinazione
	; del salto del frammento di codice corrispondente alla
	; traduzione del "case" dello switch
	; l'accesso a Jump Table viene fatto con l'indirizzamento base-indicizzato
	; l'indice va moltiplicato per 4 in quanto ogni indirizzo occupa 1 long
	; la moltiplicazione per 4 puo' essere fatta con MULU oppure
	; con uno shift a sinistra di 2 posizioni: una delle due istruzioni
	; seguenti deve essere commentata
	;mulu	#4,d0		; moltiplica per 4 mediante MULU
	lsl.l	#2,d0		; moltiplica per 4 mediante shift logico a sinistra
	move.l	#jump_table,a0
	move.l	(a0,d0.l),a1
	jmp	(a1)		; salto alla traduzione del "case"

* Casi dello switch 
sw_case_0:
	move.l	#-4,a
	bra	switch_end
	
sw_case_2:
	move.l	a,d2
	add.l	b,d2
	move.l	d2,c
;	/* questo caso va a finire (fall-through) nel caso 3 */
sw_case_3:
	move.l	c,b
	sub.l	#2,b
	bra	switch_end
		
sw_case_m1:
	move.l	x,d2
	add.l	#3,d2
	add.l	d2,b
;	/* questo caso va a finire (fall-through) nel caso 4 */
sw_case_4:
	move.l	#20,c
	move.l	b,a
	bra	switch_end
	
sw_default:
	sub.l	#1,a

switch_end:

	; calcola il valore finale di x e lo copia in memoria
	move.l	a,d2
	add.l	b,d2
	add.l	c,d2
	move.l	d2,x

fine_codice:

	end

* Original Plain C Code

#include <stdio.h>

int main(void) {

	int a = 4, b = 10, c = 20, x;

	scanf( "%d", &x );

	switch( x ) {
	
	case 0:
		a = -4;
		break;
	case 2:
		c = a + b;
		/* questo caso va a finire (fall-through) nel caso 3 */
	case 3:
		b = c - 2;
		break;
	case -1:
		b += x + 3;
		/* questo caso va a finire (fall-through) nel caso 4 */
	case 4:
		c = 20;
		a = b;
		break;
	default:
		--a;
	}
	x = a + b + c;
	printf( "%d\n", x );

	return 0;
}

/*
	Test Cases

	Test 1
	input: -2
	output: 33

	Test 2
	input: -1
	output: 44

	Test 3
	input: 0
	output: 26

	Test 4
	input: 1
	output: 33

	Test 5
	input: 2
	output: 30

	Test 6
	input: 3
	output: 42

	Test 7
	input: 4
	output: 40

	Test 8
	input: 5
	output: 33
*/

