# -------------------------------
# 	LPS Example
#
# 	Switch
#
#	Language: MIPS32-MARS
#	Style: plain MIPS32
# 	Version: Ref2-mips
# -------------------------------

# Reference Plain MIPS32 Solution 2

# Questa soluzione mostra come tradurre in ASM
# un programma C che contiene un'istruzione switch
# mediante la tecnica della "Jump Table"

# Questo esempio e' realizzato per l'assembler MARS
# Per eseguire il programma si possono usare i comandi della GUI

# Variabili
# il tipo int viene memorizzato con word
	.data	0x10010000
a:	.word	4
b:	.word	10
c:	.word	20
x:	.word	5

# Jump Table: un array di indirizzi di istruzioni
jump_table:
	.word	sw_case_m1,sw_case_0,sw_default
	.word	sw_case_2,sw_case_3,sw_case_4

# Programma
# Indica esplicitamente l'indirizzo di inizio della sezione di codice
	.text	0x400000
inizio_codice:	

	# nel programma, memorizza le variabili in registri
	# al termine copia i valori di nuovo in memoria
	lw	$s1,x
	lw	$t0,a
	lw	$t1,b
	lw	$t2,c
	
switch_beg:

	# se il valore della variabile è esterno
	# all'intervallo dei valori delle etichette-case,
	# salta al caso di default
	blt	$s1,-1,sw_default
	bgt	$s1,4,sw_default
	
	# trasformo $s1 in un indice in Jump Table, aumentandolo di 1
	add	$t3,$s1,1
	
	# leggo da Jump Table l'indirizzo dell'istruzione destinazione
	# del salto del frammento di codice corrispondente alla
	# traduzione del "case" dello switch
	# l'accesso a Jump Table viene fatto con l'indirizzamento indicizzato 
	# l'indice va moltiplicato per 4 in quanto ogni indirizzo occupa 1 word
	# la moltiplicazione per 4, può essere fatta con MULU oppure
	# con uno shift a sinistra di 2 posizioni: una delle due istruzioni
	# seguenti deve essere commentata
	#mulu	$t3,$t3,4	# moltiplica per 4 mediante MULU
	sll	$t3,$t3,2	# moltiplica per 4 mediante shift logico a sinistra
	lw	$t4,jump_table($t3)
	jr	$t4		# salto alla traduzione del "case"

# Casi dello switch 
sw_case_0:
	li	$t0,-4
	sw	$t0,a
	j	switch_end
	
sw_case_2:
	add	$t2,$t0,$t1
	sw	$t2,c
#	/* questo caso va a finire (fall-through) nel caso 3 */
sw_case_3:
	sub	$t1,$t2,2
	sw	$t1,b
	j	switch_end
		
sw_case_m1:
	add	$t3,$s1,3
	add	$t1,$t1,$t3
	sw	$t1,b
#	/* questo caso va a finire (fall-through) nel caso 4 */
sw_case_4:
	li	$t2,20
	sw	$t2,c
	move	$t0,$t1
	sw	$t0,a
	j	switch_end
	
sw_default:
	sub	$t0,$t0,1
	sw	$t0,a

switch_end:

	# calcola il valore finale di x e lo copia in memoria
	add	$s1,$t0,$t1
	add	$s1,$s1,$t2
	sw	$s1,x

fine_codice:	

# Original Plain C Code
#
# #include <stdio.h>
#
# int main(void) {
#
#	int a = 4, b = 10, c = 20, x;
#
#	scanf( "%d", &x );
#
#	switch( x ) {
#	
#	case 0:
#		a = -4;
#		break;
#	case 2:
#		c = a + b;
#		/* questo caso va a finire (fall-through) nel caso 3 */
#	case 3:
#		b = c - 2;
#		break;
#	case -1:
#		b += x + 3;
#		/* questo caso va a finire (fall-through) nel caso 4 */
#	case 4:
#		c = 20;
#		a = b;
#		break;
#	default:
#		--a;
#	}
#	x = a + b + c;
#	printf( "%d\n", x );
#
#	return 0;
# }
#
# /*
#	Test Cases
#
#	Test 1
#	input: -2
#	output: 33
#
#	Test 2
#	input: -1
#	output: 44
#
#	Test 3
#	input: 0
#	output: 26
#
#	Test 4
#	input: 1
#	output: 33
#
#	Test 5
#	input: 2
#	output: 30
#
#	Test 6
#	input: 3
#	output: 42
#
#	Test 7
#	input: 4
#	output: 40
#
#	Test 8
#	input: 5
#	output: 33
# */
